@@ -12,8 +12,9 @@
 #' @param dims factors for dimensions
 #' @param model model file or a \code{bi_model} object (if \code{x} is not a \code{libbi} object)
 #' @param type vector of types of variable to read (out of "param", "state", "noise", "obs"). This needs 'x' to be a \code{\link{libbi}} object or \code{model} to be specified
+#' @param file which file to read (if \code{x} is given as a \code{\link{libbi}} object): one of "output" (default), "init", "input", "obs"
 #' @param missval.threshold upper threshold for the likelihood
-#' @param coord_name name of coord dimension (if any)
+#' @param coord_dims any \code{coord} dimensions, given as a named list of character vectors, where each element corresponds to the variable of the same name, and the character vector are the \code{coord} dimensions
 #' @param vector if TRUE, will return results as vectors, not data.frames
 #' @param thin thinning (keep only 1/thin of samples)
 #' @param verbose if TRUE, will print variables as they are read
@@ -28,7 +29,7 @@
 #' example_output_file <- system.file(package="rbi", "example_output.nc")
 #' d <- bi_read(example_output_file)
 #' @export
-bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_name, vector, thin, verbose, clear_cache, init.to.param=FALSE)
+bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_dims, vector, thin, verbose, clear_cache, init.to.param=FALSE)
 {
   if (missing(file)) {
     nc <- bi_open(x)
@@ -41,6 +42,14 @@ bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_n
     as.integer(ifelse(missing(thin),
                       ifelse("libbi" %in% class(x), x$thin, 1), thin))
 
+  legacy_coord_dims <- NA_character_
+  if (missing(coord_dims)) {
+    coord_dims <- NULL
+  } else if (is.character(coord_dims)) {
+    legacy_coord_dims <- coord_dims
+    coord_dims <- list()
+  }
+
   if ("libbi" %in% class(x) && !is.null(x$dims)) {
     if (missing(model)) {
       model <- x$model
@@ -52,6 +61,11 @@ bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_n
     } else {
       warning("Given 'dims' will overwrite dimensions in passed libbi object")
     }
+    if (is.null(coord_dims)) {
+      coord_dims <- x$coord_dims
+    } else {
+      warning("Given 'coord_dims' will overwrite dimensions in passed libbi object")
+    }
   }
 
   all_nc_var_names <- unname(vapply(nc[["var"]], function(y) { y[["name"]] }, ""))
@@ -78,7 +92,10 @@ bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_n
 
   if (!missing(type)) {
     vars <- var_names(model, type, opt=TRUE)
+    ## remove vars that don't have an output
     vars <- grep("has_output[^=]*=[^[0-1]*0", vars, value=TRUE, invert=TRUE)
+    ## remove any other options
+    vars <- gsub("[[:space:]]*\\(.*$", "", vars)
   }
 
   if (!missing(vars)) {
@@ -136,13 +153,14 @@ bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_n
       message(date(), " ", var_name)
     }
     if (missing(vars) || var_name %in% vars) {
-      dim_names <- var_dims[["other"]][[var_name]]
-      dim_lengths <- vapply(seq_along(dim_names), function(y)
+
+      dim_var_names <- var_dims[["other"]][[var_name]]
+      dim_lengths <- vapply(seq_along(dim_var_names), function(y)
       {
         nc[["var"]][[var_name]][["dim"]][[y]][["len"]]
       }, 0)
-      names(dim_lengths) <- dim_names
-      if ("np" %in% dim_names && thin > 1) {
+      names(dim_lengths) <- dim_var_names
+      if ("np" %in% dim_var_names && thin > 1) {
         np_indices <- seq(1, dim_lengths["np"], by = thin)
         dim_lengths["np"] <- length(np_indices)
 
@@ -175,14 +193,14 @@ bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_n
 
       if ("np" %in% names(dim_lengths) && dim_lengths[["np"]] == 1) {
         dim_lengths <- dim_lengths[names(dim_lengths) != "np"]
-        dim_names <- names(dim_lengths)
+        dim_var_names <- names(dim_lengths)
       }
 
-      if (any(duplicated(dim_names))) {
-        duplicated_dim_names <- dim_names[duplicated(dim_names)]
+      if (any(duplicated(dim_var_names))) {
+        duplicated_dim_names <- dim_var_names[duplicated(dim_names)]
         for (dup_dim in duplicated_dim_names) {
-          dim_names[dim_names == dup_dim] <-
-            paste(dup_dim, seq_along(dim_names[dim_names == dup_dim]), sep = ".")
+          dim_var_names[dim_var_names == dup_dim] <-
+            paste(dup_dim, seq_along(dim_var_names[dim_var_names == dup_dim]), sep = ".")
         }
       }
 
@@ -193,20 +211,29 @@ bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_n
 
       if (!is.null(dim(all_values))) {
 
-        mav <- data.table::data.table(data.table::melt(all_values, varnames = dim_names))
+        mav <- data.table::data.table(data.table::melt(all_values, varnames = dim_var_names))
         ## remove any extraneous dimensions from melting
-        mav <- mav[, c(dim_names, "value"), with=F]
+        mav <- mav[, c(dim_var_names, "value"), with=F]
 
         ## find matching time and coord variables
         all_matching_dims <- c()
         for (var_type in names(time_coord_names)) {
-          matching_dims <- unname(unlist(var_dims[[var_type]])[unlist(var_dims[[var_type]]) %in% dim_names])
-          matching_vars <- names(var_dims[[var_type]])[vapply(var_dims[[var_type]], function(x) {any(x %in% dim_names)}, TRUE)]
+          matching_dims <- unname(unlist(var_dims[[var_type]])[unlist(var_dims[[var_type]]) %in% dim_var_names])
+          matching_vars <- names(var_dims[[var_type]])[vapply(var_dims[[var_type]], function(x) {any(x %in% dim_var_names)}, TRUE)]
           all_matching_dims <- union(all_matching_dims, matching_dims)
           if (length(matching_vars) == 1)  {
             merge_values <- ncvar_get(nc, matching_vars)
-            mav_merge <- data.table::data.table(data.table::melt(merge_values, varnames = matching_dims, value.name = time_coord_names[var_type]))
-            mav <- merge(mav_merge, mav, by = unname(matching_dims))
+            if (var_type == "coord" && !is.null(coord_dims[[var_name]])) {
+              if (length(dim(merge_values)) == 1) {
+                merge_values <- data.table::data.table(coord=merge_values)
+              }
+              colnames(merge_values) <- coord_dims[[var_name]]
+              merge_values <- apply(merge_values, 2, as.integer)
+              mav <- cbind(merge_values, mav)
+            } else {
+              mav_merge <- data.table::data.table(data.table::melt(merge_values, varnames = matching_dims, value.name = time_coord_names[var_type]))
+              mav <- merge(mav_merge, mav, by = unname(matching_dims))
+            }
           } else if (length(matching_vars) > 1) {
             stop(paste0("Found multiple matching", var_type, " variables for ", var_name, ": ", matching_vars))
           }
@@ -226,14 +253,20 @@ bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_n
         if (length(cols) > 0) setkeyv(mav, cols)
         rownames(mav) <- seq_len(nrow(mav))
 
-        if (!missing(dims) && !is.null(dims) && length(dims) == 1 && "coord" %in% colnames(mav)) {
+        if ("libbi" %in% class(x) && length(x$coord_dim) == 1 && "coord" %in% colnames(mav)) {
+          setnames(mav, "coord", x$coord_dim)
+        } else if (!missing(dims) && !is.null(dims) && length(dims) == 1 && "coord" %in% colnames(mav)) {
           setnames(mav, "coord", names(dims))
         }
 
+        if ("libbi" %in% class(x) && length(x$time_dim) == 1 && "time" %in% colnames(mav)) {
+          setnames(mav, "time", x$time_dim)
+        }
+
         for (col in colnames(mav)) {
           if (!missing(dims) && !is.null(dims) && col %in% names(dims)) {
             mav[[col]] <- factor(mav[[col]], labels = dims[[col]])
-          } else if (!(col %in% c("value", time_coord_names[c("time", "coord")]))) {
+          } else if (col %in% var_dims[["other"]][[var_name]]) {
             mav[[col]] <- mav[[col]] - 1
           }
         }
@@ -265,7 +298,7 @@ bi_read <- function(x, vars, dims, model, type, file, missval.threshold, coord_n
     }
   }
 
-  if (typeof(x) %in% c("character", "libbi")) nc_close(nc)
+  if (any(class(x) %in% c("character", "libbi"))) nc_close(nc)
 
   if ("libbi" %in% class(x) && x$use_cache &&
       (missing(file) || file == "output")) {
